"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.REMOTE_ACTION = undefined;

var _regenerator = require("babel-runtime/regenerator");

var _regenerator2 = _interopRequireDefault(_regenerator);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.gameMiddleware = gameMiddleware;

var _game = require("@cardcore/game");

var gameActions = _interopRequireWildcard(_game);

var _client = require("@cardcore/client");

var _util = require("@cardcore/util");

var _elements = require("@cardcore/elements");

var _ssbKeys = require("@streamplace/ssb-keys");

var _ssbKeys2 = _interopRequireDefault(_ssbKeys);

var _jsonStableStringify = require("json-stable-stringify");

var _jsonStableStringify2 = _interopRequireDefault(_jsonStableStringify);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } // import signalhub from "signalhub";


// this used to be a Symbol, but not supported on android?! weird.
var REMOTE_ACTION = exports.REMOTE_ACTION = "__REMOTE_ACTION";

function gameMiddleware(store) {
  var _this = this;

  var promises = new WeakMap();

  return function (next) {
    var queue = [];
    var running = false;
    var sync = true;
    var prevHash = null;

    var runNext = function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var action, me, isGameAction, ret, hash, prev, _next, signedAction, res, server, _promises$get, _promises$get2, resolve, state, nextActions, _nextActions$, playerId, notPlayerId, _action;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!running) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt("return");

              case 2:
                if (!(queue.length === 0)) {
                  _context.next = 5;
                  break;
                }

                store.dispatch((0, _client.clientPoll)());
                return _context.abrupt("return");

              case 5:
                action = queue.shift();

                if (!(!sync && action.type !== gameActions.DESYNC)) {
                  _context.next = 9;
                  break;
                }

                // if we lost sync, the only thing we accept is desync reports
                running = false;
                return _context.abrupt("return");

              case 9:
                running = true;
                me = store.getState().client.keys;
                isGameAction = !!gameActions[action.type];
                _context.next = 14;
                return next(Object.assign({}, action, {
                  _me: me && me.id,
                  prev: isGameAction ? prevHash : undefined
                }));

              case 14:
                ret = _context.sent;
                _context.next = 17;
                return (0, _util.hashState)(store.getState().game);

              case 17:
                hash = _context.sent;

                if (!action[REMOTE_ACTION]) {
                  _context.next = 23;
                  break;
                }

                prevHash = hash;
                // we just completed a remote action, assert states match
                if (sync && hash !== action.next) {
                  // very bad and extremely fatal for now - perhaps someday we recover
                  sync = false;
                  store.dispatch(gameActions.desync(me.id, store.getState().game));
                }
                _context.next = 38;
                break;

              case 23:
                if (!(gameActions[action.type] && action.type !== gameActions.DESYNC)) {
                  _context.next = 38;
                  break;
                }

                prev = prevHash;

                prevHash = hash;
                // tell everyone else the action happened and the resulting hash
                _next = hash;
                signedAction = _ssbKeys2.default.signObj(me, Object.assign({}, action, {
                  prev: prev,
                  next: _next
                }));
                _context.next = 30;
                return (0, _elements.serverFetch)("/" + encodeURIComponent(_next), {
                  method: "POST",
                  body: (0, _jsonStableStringify2.default)(signedAction),
                  headers: {
                    "content-type": "application/json"
                  }
                });

              case 30:
                res = _context.sent;

                if (!(res.status === 409)) {
                  _context.next = 38;
                  break;
                }

                sync = false;
                store.dispatch(gameActions.desync("client", store.getState().game));
                _context.next = 36;
                return res.json();

              case 36:
                server = _context.sent;

                store.dispatch(gameActions.desync("server", server.state.game));

              case 38:
                _promises$get = promises.get(action), _promises$get2 = _slicedToArray(_promises$get, 1), resolve = _promises$get2[0]; // hack, maybe should reject?

                running = false;
                state = store.getState();
                nextActions = state.game && state.game.nextActions;
                // only dequeue if a game action just happened - client actions don't count

                if (!(nextActions && nextActions.length > 0 && (gameActions[action.type] || action.type === _client.CLIENT_LOAD_STATE_DONE) && !state.client.loadingState)) {
                  _context.next = 49;
                  break;
                }

                _nextActions$ = nextActions[0], playerId = _nextActions$.playerId, notPlayerId = _nextActions$.notPlayerId, _action = _nextActions$.action;

                if (gameActions[_action.type]) {
                  _context.next = 46;
                  break;
                }

                throw new Error(_action.type + " is queued but we don't have a definition");

              case 46:
                if (!(playerId && playerId === me.id || notPlayerId && notPlayerId !== me.id // hack hack hack
                )) {
                  _context.next = 49;
                  break;
                }

                _context.next = 49;
                return store.dispatch(Object.assign({}, _action, {
                  _fromQueue: true,
                  _needsCreator: true,
                  _sender: me.id
                }));

              case 49:
                resolve(ret);
                runNext();

              case 51:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, _this);
      }));

      return function runNext() {
        return _ref.apply(this, arguments);
      };
    }();

    return function (action) {
      if (!action[REMOTE_ACTION]) {
        action = Object.assign({}, action, { _sender: store.getState().client.keys.id });
      }
      queue.push(action);
      var prom = new Promise(function (resolve, reject) {
        promises.set(action, [resolve, reject]);
      });
      setTimeout(runNext, 0);
      return prom;
    };
  };
}
//# sourceMappingURL=game-middleware.js.map