"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.createReducer = createReducer;

var _game = require("@cardcore/game");

var gameActions = _interopRequireWildcard(_game);

var _client = require("@cardcore/client");

var clientActions = _interopRequireWildcard(_client);

var _queueReducer = require("./queue-reducer");

var _queueReducer2 = _interopRequireDefault(_queueReducer);

var _util = require("@cardcore/util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// automatically find any reducer functions in the actions file and call them
var gameReducers = Object.keys(gameActions).filter(function (key) {
  return key.endsWith("Reducer");
}).map(function (key) {
  return gameActions[key];
});

var secret = function secret() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments[1];

  if (action.type === clientActions.CLIENT_GENERATE_KEY) {
    return Object.assign({}, state, _defineProperty({}, action.keys.id, action.keys));
  }

  if (action.type === clientActions.CLIENT_BOX) {
    return Object.assign({}, state, _defineProperty({}, action.id, Object.assign({}, state[action.id], {
      contents: action.contents
    })));
  }

  return state;
};

var DEFAULT_STATE = { game: {} };

function createReducer() {
  for (var _len = arguments.length, modules = Array(_len), _key = 0; _key < _len; _key++) {
    modules[_key] = arguments[_key];
  }

  var moduleReducers = modules.map(function (mod) {
    return Object.keys(mod).filter(function (key) {
      return key.endsWith("Reducer");
    }).map(function (key) {
      return mod[key];
    });
  }).reduce(function (acc, funcs) {
    return [].concat(_toConsumableArray(acc), _toConsumableArray(funcs));
  }, []);
  return function rootReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_STATE;
    var action = arguments[1];

    var startRandoSeed = state.game && state.game.randoSeed;
    if (state.game && state.game.randoSeed) {
      _util.rando.setSeed(startRandoSeed);
    }
    var reducers = { client: clientActions.clientReducer, secret: secret };
    // temporary hacky game init logic
    if (action.type === clientActions.CLIENT_LOAD_STATE_START) {
      state = Object.assign({}, state, {
        game: action.gameState
      });
    }
    // special logic to clean out the queue if we're executing a queued action
    if (state && state.game && state.game.nextActions && state.game.nextActions[0] && (action._sender === state.game.nextActions[0].playerId || action._sender !== state.game.nextActions[0].notPlayerId) && // omfg hack
    action.type === state.game.nextActions[0].action.type) {
      state = Object.assign({}, state, {
        game: Object.assign({}, state.game, {
          nextActions: state.game.nextActions.slice(1)
        })
      });
    }

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = Object.entries(reducers)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _ref = _step.value;

        var _ref2 = _slicedToArray(_ref, 2);

        var name = _ref2[0];
        var reducer = _ref2[1];

        var newState = reducer(state[name], action);
        if (state[name] !== newState) {
          state = Object.assign({}, state, _defineProperty({}, name, newState));
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    var _arr = [].concat(_toConsumableArray(gameReducers), _toConsumableArray(moduleReducers));

    for (var _i = 0; _i < _arr.length; _i++) {
      var _reducer = _arr[_i];
      state = _reducer(state, action);
      if (!state) {
        throw new Error(_reducer.name + " returned undefined");
      }
    }

    if (state.game && _util.rando.seed && state.game.randoSeed !== _util.rando.seed) {
      state = Object.assign({}, state, {
        game: Object.assign({}, state.game, {
          randoSeed: _util.rando.seed
        })
      });
    }
    if (gameActions[action.type]) {
      state = (0, _queueReducer2.default)(state, action);
    }
    _util.rando.clearSeed();
    return state;
  };
}

exports.default = createReducer();
//# sourceMappingURL=reducer.js.map